/************************************************************************
	filename: 	TS_AppliConfig_xmlHandler.h
	created:	03/12/2005
	author:		martin FLEURENT
	
	purpose:	Interface to configuration file parser
*************************************************************************/
#ifndef _TS_AppliConfig_xmlHandler_h_
#define _TS_AppliConfig_xmlHandler_h_

#include "CEGUIBase.h"
#include "CEGUIString.h"
#include "CEGUIXMLHandler.h"
#include "TS_SkinHelper.h"

#include <vector>

/*!
\brief
	Handler class used to parse the Configuration XML file.
*/
class TSAppliConfig_xmlHandler : public CEGUI::XMLHandler
{
public:

	typedef std::vector<CEGUI::String>	LayoutsStack;


	/*************************************************************************
		Construction & Destruction
	*************************************************************************/
	/*!
	\brief
		Constructor for GUILayout_xmlHandler objects
	*/
	TSAppliConfig_xmlHandler(void) {}

	/*!
	\brief
		Destructor for GUILayout_xmlHandler objects
	*/
	virtual ~TSAppliConfig_xmlHandler(void) {}

	/*************************************************************************
		SAX2 Handler overrides
	*************************************************************************/ 
	/*!
	\brief
		document processing (only care about elements, schema validates format)
	*/
    virtual void elementStart(const CEGUI::String& element, const CEGUI::XMLAttributes& attributes);

	/*************************************************************************
		Functions used by our implementation
	*************************************************************************/
	/*!
	\brief
		Return log filename
	*/
	LayoutsStack	getLayoutsList(void) const				{return d_layoutsList;}


	/*!
	\brief
		Return initial scheme filename to load
	*/
	Skin::SkinsStack	getSkinsList(void) const			{return d_skinsList;}
	
	/*!
	\brief
		Return log filename
	*/
	CEGUI::colour	getBackgroundColour(void) const				{return d_backgroundColor;}

private:
	/*************************************************************************
		Implementation Constants
	*************************************************************************/
	static const CEGUI::String AppliConfigElement;	//!< Tag name for TestaSkin Config elements.
	static const CEGUI::String LayoutsListElement;	//!< Tag name for TestaSkin Config elements.
	static const CEGUI::String SkinsListElement;		//!< Tag name for TestaSkin Config elements.
	static const CEGUI::String LayoutElement;				//!< Tag name for TestaSkin Config elements.
	static const CEGUI::String SkinElement;			  	//!< Tag name for TestaSkin Config elements.
	static const CEGUI::String BackgroundElement;   //!< Tag name for TestaSkin Config elements.
	static const char	LayoutFileAttribute[];        //!< The filename to use for loading a layout.
	static const char	SkinFontAttribute[];          //!< The filename to use for loading a default skin font.
	static const char	SkinSchemeAttribute[];        //!< The filename to use for loading a skin scheme.
	static const char	SkinAliasAttribute[];         //!< The filename to use for loading a skin alias.
	static const char	SkinLooknfeelAttribute[];     //!< The filename to use for loading a skin looknfeel.
	static const char	SkinCursorAttribute[];        //!< The name of the default skin cursor.
  static const char BackgroundColourAttribute[];   //!< Default color for the background.

	/*************************************************************************
		Implementation Data
	*************************************************************************/

	LayoutsStack d_layoutsList;
	Skin::SkinsStack d_skinsList;
	CEGUI::colour d_backgroundColor;

};


#endif	// end of guard _TS_AppliConfig_xmlHandler_h_
