/************************************************************************
  filename:   TS_AppliConfig_xmlHandler.cpp
	created:	03/12/2005
	author:		martin FLEURENT
    
  purpose:    Implements configuration file parser
*************************************************************************/
#include "TS_AppliConfig_xmlHandler.h"
#include "CEGUIExceptions.h"
#include "CEGUISystem.h"
#include "CEGUIXMLAttributes.h"
#include "CEGUIPropertyHelper.h"


/*************************************************************************
    Implementation Constants
*************************************************************************/
const CEGUI::String TSAppliConfig_xmlHandler::AppliConfigElement( (CEGUI::utf8*)"TSAppliConfig" );
const CEGUI::String TSAppliConfig_xmlHandler::LayoutsListElement( (CEGUI::utf8*)"Layouts" );
const CEGUI::String TSAppliConfig_xmlHandler::SkinsListElement( (CEGUI::utf8*)"Skins" );
const CEGUI::String TSAppliConfig_xmlHandler::LayoutElement( (CEGUI::utf8*)"Layout" );
const CEGUI::String TSAppliConfig_xmlHandler::SkinElement( (CEGUI::utf8*)"Skin" );
const CEGUI::String TSAppliConfig_xmlHandler::BackgroundElement( (CEGUI::utf8*)"Background" );
const char  TSAppliConfig_xmlHandler::LayoutFileAttribute[]       = "File";
const char  TSAppliConfig_xmlHandler::SkinFontAttribute[]         = "Font";
const char  TSAppliConfig_xmlHandler::SkinSchemeAttribute[]       = "Scheme";
const char  TSAppliConfig_xmlHandler::SkinAliasAttribute[]        = "Alias";
const char  TSAppliConfig_xmlHandler::SkinLooknfeelAttribute[]    = "Looknfeel";
const char  TSAppliConfig_xmlHandler::SkinCursorAttribute[]       = "Cursor";
const char  TSAppliConfig_xmlHandler::BackgroundColourAttribute[]  = "Colour";

void TSAppliConfig_xmlHandler::elementStart(const CEGUI::String& element, const CEGUI::XMLAttributes& attributes)
{
    // handle root AppliConfigElement element
    if (element == AppliConfigElement)
    {
			;
    }
		// handle LayoutsList element
		else if (element == LayoutsListElement)
		{
			;
		}
		// handle SkinsList element
		else if (element == SkinsListElement)
		{
			;
		}
		// handle Layout element
		else if (element == LayoutElement)
		{
			d_layoutsList.push_back(attributes.getValueAsString(LayoutFileAttribute));
		}
		// handle Skin element
		else if (element == SkinElement)
		{
			CEGUI::String font(attributes.getValueAsString(SkinFontAttribute));
			CEGUI::String scheme(attributes.getValueAsString(SkinSchemeAttribute));
			CEGUI::String alias(attributes.getValueAsString(SkinAliasAttribute));
			CEGUI::String looknfeel(attributes.getValueAsString(SkinLooknfeelAttribute));
			CEGUI::String cursor(attributes.getValueAsString(SkinCursorAttribute));
			SkinInfo* skin = new SkinInfo(font, scheme, alias, looknfeel, cursor);
			d_skinsList.push_back(skin);

		}
		// handle Background element
		else if (element == BackgroundElement)
		{
			d_backgroundColor =  CEGUI::PropertyHelper::stringToColour(attributes.getValueAsString(BackgroundColourAttribute));
		}
    // anything else is an error which *should* have already been caught by XML validation
    else
    {
        CEGUI::String message("TSAppliConfig_xmlHandler::startElement - Unexpected data was found while parsing the configuration file: '" + element + "' is unknown.");

        // throw a std::exception (because it won't try and use logger)
        throw message.c_str();
    }

}

