/************************************************************************
filename:   TS_LayoutInternal.cpp
created:    27/10/2005
author:     martin FLEURENT
*************************************************************************/
#include "TS_LayoutInternal.h"
#include "CEGUI.h"
#include "TS_SkinHelper.h"
#include "CEGUICommonFileDialog.h"




TSLayoutInternal::TSLayoutInternal():
d_toolbox(0),
d_layout(0),
d_root(0)
{	
	// set default mouse image
	CEGUI::System::getSingleton().setDefaultMouseCursor(SkinHelper::usedSkin._skinUsed->name_SkinLooknfeel, SkinHelper::usedSkin._skinUsed->name_SkinDefaultMouse);

	_Menu = new TSMenu();

	// create and add the skinselector
	_selectorSkin = new TSSkinSelector();
	_selectorLayout = new TSLayoutSelector();
	_selectorBackground = new TSBackgroundSelector();

	_wndAbout = new TSAbout();
	_wndHelp = new TSHelp();
}
TSLayoutInternal::~TSLayoutInternal()
{
	d_toolbox = 0;
	d_layout = 0;
	d_root = 0;

	delete _Menu;
	_Menu = 0;
	
	delete _selectorSkin;
	_selectorSkin = 0;

	delete _selectorLayout;
	_selectorLayout = 0;

	delete _selectorBackground;
	_selectorBackground = 0;

	delete _wndAbout;
	_wndAbout = 0;

	delete _wndHelp;
	_wndHelp = 0;
}

CEGUI::Window* TSLayoutInternal::build()
{
	using namespace CEGUI;	
	// Get window manager which we wil use for a few jobs here.
	WindowManager& winMgr = WindowManager::getSingleton();

    WindowFactoryManager& wfMgr = WindowFactoryManager::getSingleton();
//    WindowFactory* factory = wfMgr.getFactory(type);

	const Widgets& SkinWidgets = SkinHelper::usedSkin.name_Widgets;

	// create the layout root
	d_root = static_cast<StaticImage*>(winMgr.createWindow("Marti/StaticImage", "Root"));
	
	d_root->subscribeEvent(Window::EventKeyDown, Event::Subscriber(&TSLayoutInternal::handleKeyDown, this));	

	// setting
	d_root->setAreaRect( Rect(0, 0, 1, 1));
	colour BackgroundColor( 0.6f, 0.6f, 0.6f );
	d_root->setBackgroundColours( BackgroundColor);

	// disable frame and 
	d_root->setFrameEnabled(false);

	d_toolbox = static_cast<FrameWindow*>(winMgr.createWindow(SkinWidgets.FrameWindow, "Toolbox"));	
	d_toolbox->setTitleBarEnabled(false);
	d_toolbox->setSizingEnabled(false);
	d_toolbox->setCloseButtonEnabled(false);
	d_toolbox->setAlwaysOnTop(true);
	d_toolbox->setZOrderingEnabled(false);

	d_toolbox->setWindowPosition(UVector2(UDim( 0.0f, -4.0f), UDim( 0.0f, 19.0f)));
	d_toolbox->setWindowSize(UVector2(UDim( 0.0f, 200.0f), UDim( 1.0f,0.0f)));	

	d_toolbox->addChildWindow(_selectorSkin->build());
	d_toolbox->addChildWindow(_selectorLayout->build());
	d_toolbox->addChildWindow(_selectorBackground->build());
	d_root->addChildWindow(d_toolbox);

	// add the menu after the toolbox to make clean Z ordering
	d_root->addChildWindow(_Menu->build());
	
	d_root->addChildWindow(_wndAbout->build());
	d_root->addChildWindow(_wndHelp->build());

	d_layout = static_cast<DefaultWindow*>(winMgr.createWindow(SkinWidgets.DefaultWindow, "Layout"));	

	d_layout->setWindowPosition(UVector2(UDim( 0.0f, 192.0f), UDim( 0.0f, 20.0f)));
	d_layout->setWindowSize(UVector2(UDim( 1.0f, -192.0f), UDim( 1.0f,-20.0f)));
	d_root->addChildWindow(d_layout);

	/****************************************************************************************************
	Step 1: Add the common file dialog window to the main window
		      Calling CommonFileDialog::getSingleton() instantiates the common file dialog
	*****************************************************************************************************/
	d_root->addChildWindow(CEGUI::CommonFileDialog::getSingleton().getWindow());

	CEGUI::System::getSingleton().setGUISheet(d_root);

	return d_root;
}

bool TSLayoutInternal::setLayout(CEGUI::Window* rootLayout)
{
		using namespace CEGUI;	
	// Get window manager which we wil use for a few jobs here.
	WindowManager& winMgr =WindowManager::getSingleton();

	winMgr.destroyWindow(d_layout);
	

	const Widgets& SkinWidgets = SkinHelper::usedSkin.name_Widgets;


	d_layout = static_cast<DefaultWindow*>(winMgr.createWindow(SkinWidgets.DefaultWindow, "Layout"));	
	d_layout->setWindowPosition(UVector2(UDim( 0.0f, 192.0f), UDim( 0.0f, 20.0f)));
	d_layout->setWindowSize(UVector2(UDim( 1.0f, -192.0f), UDim( 1.0f,-20.0f)));
	d_layout->addChildWindow(rootLayout);

	d_root->addChildWindow(d_layout);

	return true;
}
/*
bool TSLayoutInternal::setBackgroundColour(const CEGUI::colour& backcolour)
{
		using namespace CEGUI;	
	d_root->setBackgroundColours(backcolour);

	return true;
}
*/

bool TSLayoutInternal::handleKeyDown(const CEGUI::EventArgs& e)
{
	using namespace CEGUI;
	Logger::getSingleton().logEvent("the key '' was pressed", Standard);
	// signal quit
	//d_sampleApp->setQuitting();

	// event was handled
	return true;
}
