/************************************************************************
filename:   TS_LayoutSelector.cpp
created:    22/10/2005
author:     martin FLEURENT
*************************************************************************/
#include "TS_LayoutSelector.h"
#include "CEGUI.h"
#include "TS_SkinHelper.h"
#include "CEGUICommonFileDialog.h"
#include "Tool_TestaSkin.h"


TSLayoutSelector::TSLayoutSelector()
{

}
TSLayoutSelector::~TSLayoutSelector()
{
}

CEGUI::Window* TSLayoutSelector::build()
{
	using namespace CEGUI;

	// Get window manager which we wil use for a few jobs here.
	CEGUI::WindowManager& winMgr = CEGUI::WindowManager::getSingleton();

	const Widgets& SkinWidgets = SkinHelper::usedSkin.name_Widgets;

	/*************************************************************************
	create SkinSelector frame window
	*************************************************************************/
	FrameWindow* wndLayoutSelector = static_cast<FrameWindow*>(winMgr.createWindow(SkinWidgets.FrameWindow, "Root/wndLayoutSelector"));
	//wndLayoutSelector->setClippedByParent(false);
	wndLayoutSelector->setWindowPosition(UVector2(UDim( 0.0f, 0.0f), UDim( 0.0, 0.0f)));
	wndLayoutSelector->setWindowSize(UVector2(UDim( 1.0f, 0.0f), UDim( 0.4f, 0.0f)));
	wndLayoutSelector->setWindowMaxSize(UVector2(UDim( 1.0f, 0.0f), UDim( 1.0f, -40.0f)));
	wndLayoutSelector->setWindowMinSize(UVector2(UDim( 0.0f, 150.0f), UDim( 0.2f, 0.0f)));
	wndLayoutSelector->setText("Layout");
	wndLayoutSelector->setCloseButtonEnabled(false);
	wndLayoutSelector->setDragMovingEnabled(false);
	//wndLayoutSelector->setRollupEnabled(true);

	/*************************************************************************
	create StaticText to tell the currently used layout
	*************************************************************************/
	_txtLayoutChoice = static_cast<StaticText*>(winMgr.createWindow(SkinWidgets.StaticText, "Root/wndLayoutSelector/statictextLayout"));
	wndLayoutSelector->addChildWindow(_txtLayoutChoice);
	_txtLayoutChoice->setWindowPosition(UVector2(UDim( 0.0f, 5.0f), UDim(0.0f, 30.0f)));
	_txtLayoutChoice->setWindowSize(UVector2(UDim( 1.0f, -70.0f), UDim( 0.0f, 30.0f)));
	_txtLayoutChoice->setText("Test static");
	_txtLayoutChoice->setBackgroundEnabled(false);

	/*************************************************************************
	create build in layout Button window
	*************************************************************************/
	_radiobuttonBuilIn = static_cast<RadioButton*>(winMgr.createWindow(SkinWidgets.RadioButton, "Root/wndLayoutSelector/radiobuttonBuilIn"));
	wndLayoutSelector->addChildWindow(_radiobuttonBuilIn);
	_radiobuttonBuilIn->setWindowPosition(UVector2(UDim( 0.0f, 5.0f), UDim( 0.0f, 60.0f)));
	_radiobuttonBuilIn->setWindowSize(UVector2(UDim( 0.0f, 90.0f), UDim( 0.0f, 20.0f)));
	_radiobuttonBuilIn->setID(0);
	_radiobuttonBuilIn->setSelected(true);
	_radiobuttonBuilIn->setText("Build In");

	//	 suscribe event for managing the build in listbox enable
	_radiobuttonBuilIn->subscribeEvent(ButtonBase::EventMouseClick, Event::Subscriber(&TSLayoutSelector::handleRadioButtons, this));

	/*************************************************************************
	create file layout Button window
	*************************************************************************/
	_radiobuttonFromeFile = static_cast<RadioButton*>(winMgr.createWindow(SkinWidgets.RadioButton, "Root/wndLayoutSelector/radiobuttonFromeFile"));
	wndLayoutSelector->addChildWindow(_radiobuttonFromeFile);
	_radiobuttonFromeFile->setWindowPosition(UVector2(UDim( 0.0f, 95.0f), UDim( 0.0f, 60.0f)));
	_radiobuttonFromeFile->setWindowSize(UVector2(UDim( 1.0f, -100.0f), UDim( 0.0f, 20.0f)));
	_radiobuttonFromeFile->setID(1);
	_radiobuttonFromeFile->setText("Frome file");

	//	 suscribe event for managing the build in listbox enable
	_radiobuttonFromeFile->subscribeEvent(ButtonBase::EventMouseClick, Event::Subscriber(&TSLayoutSelector::handleRadioButtons, this));

	/*************************************************************************
	create Add Button window
	*************************************************************************/
	_buttonXmlAdd = static_cast<ButtonBase*>(winMgr.createWindow(SkinWidgets.Button, "Root/wndLayoutSelector/buttonXmlAdd"));
	wndLayoutSelector->addChildWindow(_buttonXmlAdd);
	_buttonXmlAdd->setWindowPosition(UVector2(UDim( 0.0f, 5.0f), UDim( 0.0f, 80.0f)));
	_buttonXmlAdd->setWindowSize(UVector2(UDim( 0.0f, 60.0f), UDim( 0.0f, 25.0f)));
	_buttonXmlAdd->setText("Add");

	//	 suscribe event for managing the build in listbox enable
	_buttonXmlAdd->subscribeEvent(ButtonBase::EventMouseClick, Event::Subscriber(&TSLayoutSelector::handleAddXmlLayout, this));

	_buttonXmlAdd->hide();

	/*************************************************************************
	create Del Button window
	*************************************************************************/
	_buttonXmlDel = static_cast<ButtonBase*>(winMgr.createWindow(SkinWidgets.Button, "Root/wndLayoutSelector/buttonXmlDel"));
	wndLayoutSelector->addChildWindow(_buttonXmlDel);
	_buttonXmlDel->setWindowPosition(UVector2(UDim( 0.0f, 65.0f), UDim( 0.0f, 80.0f)));
	_buttonXmlDel->setWindowSize(UVector2(UDim( 0.0f, 60.0f), UDim( 0.0f, 25.0f)));
	_buttonXmlDel->setText("Del");

	//	 suscribe event for managing the build in listbox enable
	_buttonXmlDel->subscribeEvent(ButtonBase::EventMouseClick, Event::Subscriber(&TSLayoutSelector::handleDelXmlLayout, this));

	_buttonXmlDel->hide();

	/*************************************************************************
	create BuiltIn Layouts Listbox window
	*************************************************************************/
	_wndBuildInLayoutListbox = static_cast<Listbox*>(winMgr.createWindow(SkinWidgets.Listbox, "Root/wndLayoutSelector/wndBuildInLayoutListbox"));
	wndLayoutSelector->addChildWindow(_wndBuildInLayoutListbox);
	_wndBuildInLayoutListbox->setWindowPosition(UVector2(UDim( 0.0f, 5.0f), UDim( 0.0f, 100.0f)));
	_wndBuildInLayoutListbox->setWindowSize(UVector2(UDim( 1.0f, -10.0f), UDim( 1.0f, -132.0f)));
	_wndBuildInLayoutListbox->setText("Choose a Layout");

	//	 suscribe event for managing the build in listbox enable
	//_wndBuildInLayoutListbox->subscribeEvent(ButtonBase::EventMouseClick, Event::Subscriber(&TSLayoutSelector::handleRadioButtons, this));

	/*************************************************************************
	create xml Layouts Listbox window
	*************************************************************************/
	_wndXmlLayoutListbox = static_cast<Listbox*>(winMgr.createWindow(SkinWidgets.Listbox, "Root/wndLayoutSelector/wndXmlLayoutListbox"));
	wndLayoutSelector->addChildWindow(_wndXmlLayoutListbox);
	_wndXmlLayoutListbox->setWindowPosition(UVector2(UDim( 0.0f, 5.0f), UDim( 0.0f, 100.0f)));
	_wndXmlLayoutListbox->setWindowSize(UVector2(UDim( 1.0f, -10.0f), UDim( 1.0f, -132.0f)));
	_wndXmlLayoutListbox->setText("Choose a Layout");

	//_wndXmlLayoutListbox->setEnabled(false);
	_wndXmlLayoutListbox->hide();

	// suscribe event for managing the build in listbox enable
	//_wndXmlLayoutListbox->subscribeEvent(Listbox::EventSelectionChanged, Event::Subscriber(&TSLayoutSelector::handleXmlLayoutsListbox, this));

	/*************************************************************************
	create items for the SelectorListbox
	*************************************************************************/
	// Marti
	MyListItem * itmLayoutStaticTest = new MyListItem("Static test");
	_wndBuildInLayoutListbox->addItem(itmLayoutStaticTest);
	itmLayoutStaticTest->setTooltipText("StaticTest");
	itmLayoutStaticTest->setID(0);
	itmLayoutStaticTest->setSelected(true);

	/*************************************************************************
	create OK Button window
	*************************************************************************/
	_buttonLoad = static_cast<ButtonBase*>(winMgr.createWindow(SkinWidgets.Button, "Root/wndLayoutSelector/buttonLoad"));
	wndLayoutSelector->addChildWindow(_buttonLoad);
	_buttonLoad->setWindowPosition(UVector2(UDim( 1.0f, -65.0f), UDim( 1.0f, -35.0f)));
	_buttonLoad->setWindowSize(UVector2(UDim( 0.0f, 60.0f), UDim( 0.0f, 30.0f)));
	_buttonLoad->setText("Load");

	//	 suscribe event for managing the build in listbox enable
	//_buttonLoad->subscribeEvent(ButtonBase::EventMouseClick, Event::Subscriber(&TSLayoutSelector::handleLoadLayout, this));


	/****************************************************************************************************
	Step 2: Register the function that must be called in case the common file dialog has been closed by:
	- the Ok button
	- the Cancel button
	- the Window close button ('X')
	Use the FrameWindow::EventHidden event
	*****************************************************************************************************/
	CEGUI::CommonFileDialog::getSingleton().getWindow()->subscribeEvent (CEGUI::FrameWindow::EventHidden, CEGUI::Event::Subscriber(&TSLayoutSelector::handleFileDialogOutput, this));



	return wndLayoutSelector;
}


void TSLayoutSelector::manageDelXmlState()
{
	using namespace CEGUI;
		String state("");
		uint childCount = _wndXmlLayoutListbox->getItemCount();
		// can del only if there is at least one
		if (childCount > 0)
		{
			_buttonXmlDel->enable();
			state = "enable";
		}
		else
		{
			_buttonXmlDel->disable();
			state = "disable";
		}
		//Logger::getSingleton().logEvent(state + "_________________STAMP________________________", CEGUI::Standard);
	
}

bool TSLayoutSelector::fillXmlLayoutsList(const TSAppliConfig_xmlHandler::LayoutsStack layoutsList)
{
		
	using namespace CEGUI;

	String layoutPath;
	String::size_type nameEnd;
	String::size_type nameStart;
	String layoutName;

	for (size_t index = 0; index < layoutsList.size(); ++index)
	{
		// hacky name construction
		layoutPath = layoutsList[index];
		nameEnd = layoutPath.rfind('.');
		nameStart = layoutPath.rfind('/');

		layoutName = layoutPath;
		layoutName.erase(nameEnd, layoutName.size() - nameEnd);
		layoutName.erase(0, nameStart + 1);


	  //create xml layout list item and add it to the list
	  MyListItem * itmLayout = new MyListItem(layoutName);
	  _wndXmlLayoutListbox->addItem(itmLayout);
		itmLayout->setTooltipText(layoutPath);
	  itmLayout->setID(index);
	  itmLayout->setText(layoutName);

	}
	return true;
}

bool TSLayoutSelector::handleRadioButtons(const CEGUI::EventArgs& e)
{
	using namespace CEGUI;
	// Get window manager which we wil use for a few jobs here.
	WindowManager& winMgr = WindowManager::getSingleton();

	// activate or not the listbox depend of the radio button selected

	//FrameWindow* wndSkinSelector = static_cast<FrameWindow*>(winMgr.getWindow("Root/wndSkinSelector"));

	// we want build in layout
	if( _radiobuttonBuilIn->isSelected())
	{
		// manage lisbox visibility depend the layout type we want
		//_wndBuildInLayoutListbox->enable();
		_wndBuildInLayoutListbox->show();

		//_wndXmlLayoutListbox->disable();
		_wndXmlLayoutListbox->hide();
		_buttonXmlAdd->hide();
		_buttonXmlDel->hide();

	}
	// we want layout frome file
	else
	{	
		// manage lisbox visibility depend the layout type we want
		//_wndBuildInLayoutListbox->disable();
		_wndBuildInLayoutListbox->hide();

		//_wndXmlLayoutListbox->enable();
		_wndXmlLayoutListbox->show();
		_buttonXmlAdd->show();
		_buttonXmlDel->show();

		manageDelXmlState();
	}

	// event was handled
	return true;
}

bool TSLayoutSelector::handleAddXmlLayout(const CEGUI::EventArgs& e)
{
	using namespace CEGUI;

	// Get window manager which we wil use for a few jobs here.
	//WindowManager& winMgr = WindowManager::getSingleton();

	// extract the type of layout wanted
	//uint layoutType = _radiobuttonBuilIn->getSelectedButtonInGroup()->getID();

	// want to load an xml layout file
	//if(layoutType == 1)
	//{
		/****************************************************************************************************
		Step 3a: Call the file dialog with the input data
		*****************************************************************************************************/
		CommonFileDialogInput inputDialog;
		inputDialog.setHandle (1);										// Set the handle to determine the originator
		inputDialog.setFilter ("Layout (*.layout)|*.layout|All files|*.*|");	// Extensions
		inputDialog.setOpenFileDialog (true);								// Load mode
		inputDialog.setDefaultExtension ("*.layout");						// Zip is the first one to display

		CommonFileDialog::getSingleton().openDialog(inputDialog);	// Open the common file dialog
	//}

	// event was handled
	return true;
}

bool TSLayoutSelector::handleDelXmlLayout(const CEGUI::EventArgs& e)
{
	using namespace CEGUI;

	// Get window manager which we wil use for a few jobs here.
	WindowManager& winMgr = WindowManager::getSingleton();

	// extract the selected item ID
	uint itemSelected = 0;
	ListboxItem* selectedItem =  _wndXmlLayoutListbox->getFirstSelectedItem();

	// exit if nothing selected
	if(selectedItem !=0)
	{
		_wndXmlLayoutListbox->removeItem(selectedItem);

		manageDelXmlState();
	}
	// event was handled
	return true;
}

bool TSLayoutSelector::handleFileDialogOutput(const CEGUI::EventArgs& args)
{

	using namespace CEGUI;
	/****************************************************************************************************
	Step 4: Handle the returned output
	*****************************************************************************************************/
	CommonFileDialogOutput result = CommonFileDialog::getSingleton().getResult();

	if( result.getAction() != ACTION_CANCEL && result.isFileExisting())
	{
		/*************************************************************************
		create items for the SelectorListbox
		*************************************************************************/
		MyListItem * itmLayoutStaticTest = new MyListItem(result.getFileName());
		itmLayoutStaticTest->setTooltipText(result.getFullQualifiedFileName());
		_wndXmlLayoutListbox->addItem(itmLayoutStaticTest);
		//_wndXmlLayoutListbox->clearAllSelections();
		//itmLayoutStaticTest->setSelected(true);
		
		manageDelXmlState();
	}

	return true;
}

bool TSLayoutSelector::handleLoadLayout(const CEGUI::EventArgs& e)
{
	using namespace CEGUI;

	// Get window manager which we wil use for a few jobs here.
	WindowManager& winMgr = WindowManager::getSingleton();

	// extract the selected item ID
	uint itemSelected = 0;
	ListboxItem* selectedItem =  _wndXmlLayoutListbox->getFirstSelectedItem();

	// exit if nothing selected
	if(selectedItem !=0)
		itemSelected = selectedItem->getID();
	else
		return false;

	// set default mouse image
	System::getSingleton().setDefaultMouseCursor(SkinHelper::usedSkin._skinUsed->name_SkinLooknfeel, SkinHelper::usedSkin._skinUsed->name_SkinDefaultMouse);

	// fireEvent( LayoutChanged );
	//d_toolbox->setLayout(d_layouttotest->build(selectedItem->getTooltipText()));
	_txtLayoutChoice->setText(selectedItem->getText());

	// event was handled
	return true;
}

