/************************************************************************
filename:   Tool_TestaSkin.cpp
created:    22/10/2005
author:     martin FLEURENT
*************************************************************************/
#include "Tool_TestaSkin.h"
#include "CEGUI.h"
#include "CEGUIXMLParser.h"
#include "CEGuiBaseApplication.h"
#include "falagard/CEGUIFalWidgetLookManager.h"
#include "CEGUICommonFileDialog.h"
#include "TS_AppliConfig_xmlHandler.h"

#include "TS_SkinSelector.h"
#if defined( __WIN32__ ) || defined( _WIN32 )
#define WIN32_LEAN_AND_MEAN
#define NOMINMAX
#include "windows.h"

int APIENTRY WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine,int nCmdShow)
#else
int main(int argc, char *argv[])
#endif
{
	// This is a basic start-up for the sample application which is
	// object orientated in nature, so we just need an instance of
	// the CEGuiSample based object and then tell that sample application
	// to run.  All of the samples will use code similar to this in the
	// main/WinMain function.
	TestaSkin app;
	return app.run();
}

const CEGUI::String TestaSkin::AppliConfigFile( (CEGUI::utf8*)"../datafiles/configs/TestaSkin.config" );
const CEGUI::String TestaSkin::AppliConfigSchema( (CEGUI::utf8*)"../datafiles/configs/TestaSkinConfig.xsd" );

/*************************************************************************
Sample specific initialisation goes here.
*************************************************************************/
bool TestaSkin::initialiseSample()
{
	using namespace CEGUI;

	// Get window manager which we wil use for a few jobs here.
	WindowManager& winMgr = WindowManager::getSingleton();

	//CEGUI::LoggingLevel::Insane
	Logger::getSingleton().setLoggingLevel(CEGUI::Standard);

  // create handler object
  TSAppliConfig_xmlHandler handler;

	// do parsing of config xml file
	try
	{
		CEGUI::System::getSingleton().getXMLParser()->parseXMLFile(handler, AppliConfigFile, AppliConfigSchema, "");
	}
	catch(...)
	{

		Logger::getSingleton().logEvent("TestaSkin::Config - loading of config from file '" + AppliConfigFile +"' failed.", Errors);
        
    throw;
	}

	SkinHelper::usedSkin.init(handler.getSkinsList());

	FontManager::getSingleton().createFont(SkinHelper::usedSkin._skinUsed->name_file_Font);
	

// create an instance of the console class.
	
	d_toolbox = new TSLayoutInternal();
	d_toolbox->build();
	//Logger::getSingleton().logEvent("_________________STAMP________________________", Standard);

	// make setting accoeding to the config file
	d_toolbox->_selectorLayout->fillXmlLayoutsList(handler.getLayoutsList());
	d_toolbox->_selectorSkin->fillSkinsList(SkinHelper::usedSkin._skinsList);
	d_toolbox->_selectorBackground->setColour(handler.getBackgroundColour());

	SkinHelper::usedSkin.setSkin(0);

	d_layouttotest = new TSLayoutToTest();
	

	d_layoutname = "StaticTest";
	setLayoutToTest();

	StaticText* lblSkinChoice = static_cast<StaticText*>(WindowManager::getSingleton().getWindow("Root/wndSkinSelector/statictextUsedSkin"));
	lblSkinChoice->setText(SkinHelper::usedSkin._skinUsed->name_SkinLooknfeel);

	suscribeEvents();

	// success!
	return true;
}

void TestaSkin::suscribeEvents(void)
{
	using namespace CEGUI;
	// suscribe event for key press for the root window
	
	//CEGUI::System::getSingleton().getGUISheet()->
	//	subscribeEvent(Window::EventKeyDown, Event::Subscriber(&TestaSkin::handleKeyDown, this));

	// suscribe event for the exit item of the file menu
	WindowManager::getSingleton().getWindow("Root/MenuBar/File/popFile/Exit")->
		subscribeEvent(MenuItem::EventClicked, Event::Subscriber(&TestaSkin::handleQuit, this));
	
	// suscribe event for the exit item of the file menu
	WindowManager::getSingleton().getWindow("Root/Menubar/What/popWhat/About")->
		subscribeEvent(MenuItem::EventClicked, Event::Subscriber(&TestaSkin::handleAbout, this));

	// suscribe event for the exit item of the file menu
	WindowManager::getSingleton().getWindow("Root/Menubar/What/popWhat/Help")->
		subscribeEvent(MenuItem::EventClicked, Event::Subscriber(&TestaSkin::handleHelp, this));

	// suscribe event for the change skin OK button
	WindowManager::getSingleton().getWindow("Root/wndSkinSelector/buttonOK")->
		subscribeEvent(ButtonBase::EventMouseClick, Event::Subscriber(&TestaSkin::handleChangeSkinOK, this));
	
	// temp hack until i make ToolPane and ToolBox widgets
		// suscribe event for the change skin OK button
	WindowManager::getSingleton().getWindow("Root/wndLayoutSelector/buttonLoad")->
		subscribeEvent(ButtonBase::EventMouseClick, Event::Subscriber(&TestaSkin::handleLoadLayout, this));
	
	//System::getSingleton().subscribeEvent(System:,  &TestaSkin::handleKeyDown, this);
	
}

/*************************************************************************
Cleans up resources allocated in the initialiseSample call.
*************************************************************************/
void TestaSkin::cleanupSample()
{
	delete d_toolbox;
	delete d_layouttotest;
}

bool TestaSkin::handleQuit(const CEGUI::EventArgs& e)
{
	// signal quit
	d_sampleApp->setQuitting();

	// event was handled
	return true;
}

bool TestaSkin::handleAbout(const CEGUI::EventArgs& e)
{	
	using namespace CEGUI;

	// Get window manager which we wil use for a few jobs here.
	WindowManager& winMgr = WindowManager::getSingleton();
		FrameWindow* wndAbout = static_cast<FrameWindow*>(winMgr.getWindow("Root/wndAbout"));

		if(wndAbout != 0)
		{
			//
			wndAbout->show();
			wndAbout->setModalState(true);
		}

	// event was handled
	return true;
}

bool TestaSkin::handleHelp(const CEGUI::EventArgs& e)
{	
	using namespace CEGUI;

	// Get window manager which we wil use for a few jobs here.
	WindowManager& winMgr = WindowManager::getSingleton();
		FrameWindow* wndHelp = static_cast<FrameWindow*>(winMgr.getWindow("Root/wndHelp"));

		if(wndHelp != 0)
		{
			//
			wndHelp->show();
			wndHelp->setModalState(true);
		}

	// event was handled
	return true;
}

bool TestaSkin::handleChangeSkinOK(const CEGUI::EventArgs& e)
{
	using namespace CEGUI;


	Listbox* SkinChoice = static_cast<Listbox*>(WindowManager::getSingleton().getWindow("Root/wndSkinSelector/listboxSelector"));
	
	uint itemSelected = 0;
	ListboxItem* selectedItem =  SkinChoice->getFirstSelectedItem();

	// exit if nothing selected
	if(selectedItem !=0)
			itemSelected = selectedItem->getID();
	else
		return false;

	SkinHelper::usedSkin.setSkin(itemSelected);

	// set default mouse image
	System::getSingleton().setDefaultMouseCursor(SkinHelper::usedSkin._skinUsed->name_SkinLooknfeel, SkinHelper::usedSkin._skinUsed->name_SkinDefaultMouse);

	setLayoutToTest();

	StaticText* lblSkinChoice = static_cast<StaticText*>(WindowManager::getSingleton().getWindow("Root/wndSkinSelector/statictextUsedSkin"));
	lblSkinChoice->setText(SkinHelper::usedSkin._skinUsed->name_SkinLooknfeel);

	// event was handled
	return true;
}

bool TestaSkin::handleLoadLayout(const CEGUI::EventArgs& e)
{
	using namespace CEGUI;

	// Get window manager which we wil use for a few jobs here.
	WindowManager& winMgr = WindowManager::getSingleton();

	// extract the selected item ID
	uint itemSelected = 0;
	Listbox* listboxLayoutChoice = 0;
	ListboxItem* selectedItem = 0;

	// extract the type of layout wanted
	RadioButton* radiobuttonLayoutType = static_cast<RadioButton*>(WindowManager::getSingleton().getWindow("Root/wndLayoutSelector/radiobuttonBuilIn"));
	uint layoutType = radiobuttonLayoutType->getSelectedButtonInGroup()->getID();

	// want to load an xml layout file
	if(layoutType == 1)
	{
		listboxLayoutChoice = static_cast<Listbox*>(WindowManager::getSingleton().getWindow("Root/wndLayoutSelector/wndXmlLayoutListbox"));
	}
	// want to load an xml layout file
	else
	{		
		listboxLayoutChoice = static_cast<Listbox*>(WindowManager::getSingleton().getWindow("Root/wndLayoutSelector/wndBuildInLayoutListbox"));
	}

	selectedItem =  listboxLayoutChoice->getFirstSelectedItem();

	// exit if nothing selected
	if(selectedItem !=0)
		itemSelected = selectedItem->getID();
	else
		return false;

	// set default mouse image
	System::getSingleton().setDefaultMouseCursor(SkinHelper::usedSkin._skinUsed->name_SkinLooknfeel, SkinHelper::usedSkin._skinUsed->name_SkinDefaultMouse);

	// fireEvent( LayoutChanged );
	d_layoutname = selectedItem->getTooltipText();
	setLayoutToTest();

	StaticText* lblLayoutChoice = static_cast<StaticText*>(WindowManager::getSingleton().getWindow("Root/wndLayoutSelector/statictextLayout"));
	lblLayoutChoice->setText(selectedItem->getText());

	// event was handled
	return true;
}

bool TestaSkin::setLayoutToTest()
{
	if ( d_layoutname == "StaticTest")
		d_toolbox->setLayout(d_layouttotest->build());
	else
		d_toolbox->setLayout(d_layouttotest->build(d_layoutname));

	return true;
}
